# -*- coding: utf-8 -*-
#
# File: googlevisualization.py
#
# Copyright (c) 2009 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'

from zope.formlib import form
from zope.interface import implements
from zope.schema import getFields
from zope.schema import Text, TextLine

from dictionaryproperty import DictionaryProperty

from Products.Five.formlib.formbase import EditForm

from base import IBaseField, BaseField

class IGooglevisualizationField(IBaseField):
    """
    Google chart field schema
    """
    jssettings = Text(title=u'Javascript settings',
                      description=u'Google Vizualization code',
                      default=u"""
google.load('visualization', '1', {packages:['orgchart']});
google.setOnLoadCallback(gvisudata_drawChart);
var gvisudata;

function gvisudata_drawChart() {
gvisudata = new google.visualization.DataTable();
gvisudata.addColumn('string', 'Name');
gvisudata.addColumn('string', 'Manager');
gvisudata.addColumn('string', 'ToolTip');
gvisudata_getCells();
var gvisudata_chart = new google.visualization.OrgChart(document.getElementById('gvisudata_div'));
gvisudata_chart.draw(gvisudata, {allowHtml:true});
}
""",
                      required=False)
    chartid = TextLine(title=u'Chart id',
                      description=u'Used to name the javascript variable/functions and the DIV element',
                      required=True,
                      default=u'gvisudata')

class GooglevisualizationField(BaseField):
    """ GooglevisualizationField allows to render a datatable using the Google
    Visulization tools.
    The field value is supposed to be an array of array containing the values
    each columns declared in the Google Vizualization javascript code.
    Example:
    if columns declaration is:
    gvisudata.addColumn('string', 'Name');
    gvisudata.addColumn('string', 'Manager');
    gvisudata.addColumn('string', 'ToolTip');
    (typical case when using orgchart package)
    then the field value must be like:
    [['\'Mike\',\'Mike<div style="color:red; font-style:italic">President</div>\'', "''", "'The pres'"],
    ["'Tim'", "'Mike'", "'vp'"],
    ["'Tom'", "'Mike'", "'chief'"]]
    Notes:
    - strings must be enclosed in quotes (as they will be inserted in JS code)
    - when editing the field value from the form, the rows are separated with a newline,
    and the cells are separated with a pipe:
    'Mike','Mike<div style="color:red; font-style:italic">President</div>'|''|'The pres'
    'Tim'|'Mike'|'vp'
    'Tom'|'Mike'|'chief'
    
    More information about Google Visualization javascript APIs:
    http://code.google.com/intl/en/apis/visualization/documentation/
    """
    implements(IGooglevisualizationField)

    def validate(self, submittedValue):
        """
        """
        errors=[]
        # no validation needed
        return errors
    
    def processInput(self, submittedValue):
        """
        """
        lines = submittedValue.replace('\r', '').split('\n')
        datatable = []
        for l in lines:
            datatable.append(l.split('|'))
        return datatable
    
    def jscode(self, datatable):
        """ return Google visualization js code
        """
        js = self.jssettings + "\n"
        js = js + "function " + self.chartid + "_getCells() {\n"
        js = js + self.chartid+".addRows(" + str(len(datatable)) + ");\n"
        i = 0
        for row in datatable:
            j = 0
            for cell in row:
                js = js + self.chartid+".setCell(" + str(i) + ", " + str(j) + ", " + cell + ");\n"
                j = j + 1
            i = i + 1
        js = js + "}"
        return js

    
for f in getFields(IGooglevisualizationField).values():
    setattr(GooglevisualizationField, f.getName(), DictionaryProperty(f, 'parameters'))

class SettingForm(EditForm):
    """
    """
    form_fields = form.Fields(IGooglevisualizationField)
    



# -*- coding: utf-8 -*-
#
# File: PlominoHidewhen.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.CMFPlomino.config import *

##code-section module-header #fill in your manual code here
##/code-section module-header

schema = Schema((

    StringField(
        name='id',
        widget=StringField._properties['widget'](
            label="Id",
            description="The hide-when id",
            label_msgid='CMFPlomino_label_hidewhen_id',
            description_msgid='CMFPlomino_help_hidewhen_id',
            i18n_domain='CMFPlomino',
        ),
    ),
    TextField(
        name='Formula',
        widget=TextAreaWidget(
            label="Formula",
            description="If returning True, the block will be hidden.",
            label_msgid='CMFPlomino_label_HidewhenFormula',
            description_msgid='CMFPlomino_help_HidewhenFormula',
            i18n_domain='CMFPlomino',
        ),
    ),
),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

PlominoHidewhen_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class PlominoHidewhen(BaseContent, BrowserDefaultMixin):
    """
    """
    security = ClassSecurityInfo()
    implements(interfaces.IPlominoHidewhen)

    meta_type = 'PlominoHidewhen'
    _at_rename_after_creation = True

    schema = PlominoHidewhen_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    security.declarePublic('at_post_edit_script')
    def at_post_edit_script(self):
        """
        """
        self.cleanFormulaScripts("hidewhen_"+self.getParentNode().id+"_"+self.id)


registerType(PlominoHidewhen, PROJECTNAME)
# end of class PlominoHidewhen

##code-section module-footer #fill in your manual code here
##/code-section module-footer






# -*- coding: utf-8 -*-
#
# File: PlominoForm.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces
from Products.ATContentTypes.content.folder import ATFolder
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.CMFPlomino.config import *
from Products.CMFPlomino.PlominoUtils import PlominoTranslate, DateToString

##code-section module-header #fill in your manual code here
import sys
import re
import simplejson as json

from Products.CMFCore.utils import getToolByName

import PlominoDocument
from PlominoDocument import TemporaryDocument

##/code-section module-header

schema = Schema((

    StringField(
        name='id',
        widget=StringField._properties['widget'](
            label="Id",
            description="The form id",
            label_msgid='CMFPlomino_label_FormId',
            description_msgid='CMFPlomino_help_FormId',
            i18n_domain='CMFPlomino',
        ),
    ),
    TextField(
        name='onCreateDocument',
        widget=TextAreaWidget(
            label="On create document",
            description="Action to take when the document is created",
            label_msgid='CMFPlomino_label_onCreateDocument',
            description_msgid='CMFPlomino_help_onCreateDocument',
            i18n_domain='CMFPlomino',
        ),
        schemata="Events",
    ),
    TextField(
        name='onOpenDocument',
        widget=TextAreaWidget(
            label="On open document",
            description="Action to take when the document is opned",
            label_msgid='CMFPlomino_label_onOpenDocument',
            description_msgid='CMFPlomino_help_onOpenDocument',
            i18n_domain='CMFPlomino',
        ),
        schemata="Events",
    ),
    TextField(
        name='onSaveDocument',
        widget=TextAreaWidget(
            label="On save document",
            description="Action to take when saving the document",
            label_msgid='CMFPlomino_label_onSaveDocument',
            description_msgid='CMFPlomino_help_onSaveDocument',
            i18n_domain='CMFPlomino',
        ),
        schemata="Events",
    ),
    TextField(
        name='onDeleteDocument',
        widget=TextAreaWidget(
            label="On delete document",
            description="Action to take before deleting the document",
            label_msgid='CMFPlomino_label_onDeleteDocument',
            description_msgid='CMFPlomino_help_onDeleteDocument',
            i18n_domain='CMFPlomino',
        ),
        schemata="Events",
    ),
    TextField(
        name='beforeCreateDocument',
        widget=TextAreaWidget(
            label="Before document creation",
            description="Action to take when opening a blank form",
            label_msgid='CMFPlomino_label_beforeCreateDocument',
            description_msgid='CMFPlomino_help_beforeCreateDocument',
            i18n_domain='CMFPlomino',
        ),
        schemata="Events",
    ),
    TextField(
        name='FormLayout',
        widget=RichWidget(
            label="Form layout",
            description="The form layout. text with 'Plominofield' style correspond to the contained field elements",
            label_msgid='CMFPlomino_label_FormLayout',
            description_msgid='CMFPlomino_help_FormLayout',
            i18n_domain='CMFPlomino',
        ),
        default_output_type="text/html",
    ),
    TextField(
        name='DocumentTitle',
        widget=TextAreaWidget(
            label="Document title formula",
            description="Compute the document title",
            label_msgid='CMFPlomino_label_DocumentTitle',
            description_msgid='CMFPlomino_help_DocumentTitle',
            i18n_domain='CMFPlomino',
        ),
        schemata="Parameters",
    ),
    StringField(
        name='ActionBarPosition',
        default="TOP",
        widget=SelectionWidget(
            label="Position of the action bar",
            description="Select the position of the action bar",
            label_msgid='CMFPlomino_label_ActionBarPosition',
            description_msgid='CMFPlomino_help_ActionBarPosition',
            i18n_domain='CMFPlomino',
        ),
        schemata="Parameters",
        vocabulary=[["TOP", "At the top of the page"], ["BOTTOM", "At the bottom of the page"], ["BOTH", "At the top and at the bottom of the page "]],
    ),
    BooleanField(
        name='HideDefaultActions',
        default="0",
        widget=BooleanField._properties['widget'](
            label="Hide default actions",
            description="Edit, Save, Delete, Close actions will not be displayed in the action bar",
            label_msgid='CMFPlomino_label_HideDefaultActions',
            description_msgid='CMFPlomino_help_HideDefaultActions',
            i18n_domain='CMFPlomino',
        ),
        schemata="Parameters",
    ),
    BooleanField(
        name='HideInMenu',
        default="0",
        widget=BooleanField._properties['widget'](
            label="Hide in menu",
            description="It will not appear in the database main menu",
            label_msgid='CMFPlomino_label_HideInMenu',
            description_msgid='CMFPlomino_help_HideInMenu',
            i18n_domain='CMFPlomino',
        ),
        schemata="Parameters",
    ),
    BooleanField(
        name='isSearchForm',
        default="0",
        widget=BooleanField._properties['widget'](
            label="Search form",
            description="A search form is only used to search documents, it cannot be saved.",
            label_msgid='CMFPlomino_label_SearchForm',
            description_msgid='CMFPlomino_help_SearchForm',
            i18n_domain='CMFPlomino',
        ),
        schemata="Parameters",
    ),
    BooleanField(
        name='isPage',
        default="0",
        widget=BooleanField._properties['widget'](
            label="Page",
            description="A page cannot be saved and does not provide any action bar. It can be useful to build a welcome page, explanations, reports, navigation, etc.",
            label_msgid='CMFPlomino_label_isPage',
            description_msgid='CMFPlomino_help_isPage',
            i18n_domain='CMFPlomino',
        ),
        schemata="Parameters",
    ),
    StringField(
        name='SearchView',
        widget=StringField._properties['widget'](
            label="Search view",
            description="View used to display the search results",
            label_msgid='CMFPlomino_label_SearchView',
            description_msgid='CMFPlomino_help_SearchView',
            i18n_domain='CMFPlomino',
        ),
        schemata="Parameters",
    ),
    TextField(
        name='SearchFormula',
        widget=TextAreaWidget(
            label="Search formula",
            description="Leave blank to use default Zcatalog search",
            label_msgid='CMFPlomino_label_SearchFormula',
            description_msgid='CMFPlomino_help_SearchFormula',
            i18n_domain='CMFPlomino',
        ),
        schemata="Parameters",
    ),
    IntegerField(
        name='Position',
        widget=IntegerField._properties['widget'](
            label="Position",
            label_msgid="CMFPlomino_label_Position",
            description="Position in menu",
            description_msgid="CMFPlomino_help_Position",
            i18n_domain='CMFPlomino',
        ),
        schemata="Parameters",
    ),
    BooleanField(
        name='isDynamicHidewhen',
        default=False,
        widget=BooleanField._properties['widget'](
            label="Dynamic Hide-when",
            description="Hide-when are evaluated dynamically when the user enters information",
            label_msgid='CMFPlomino_label_isDynamicHidewhen',
            description_msgid='CMFPlomino_help_isDynamicHidewhen',
            i18n_domain='CMFPlomino',
        ),
        schemata="Parameters",
    ),
),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

PlominoForm_schema = getattr(ATFolder, 'schema', Schema(())).copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class PlominoForm(ATFolder):
    """
    """
    security = ClassSecurityInfo()
    implements(interfaces.IPlominoForm)

    meta_type = 'PlominoForm'
    _at_rename_after_creation = True

    schema = PlominoForm_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    security.declareProtected(CREATE_PERMISSION, 'createDocument')
    def createDocument(self,REQUEST):
        """create a document using the forms submitted content
        """
        db = self.getParentDatabase()
        
        is_childform = False
        parent_field = REQUEST.get("Plomino_Parent_Field", None)
        parent_form = REQUEST.get("Plomino_Parent_Form", None)
        if parent_field is not None:
            is_childform = True
            
        # validate submitted values
        errors=self.validateInputs(REQUEST)
        if len(errors)>0:
            if is_childform:
                return """<script>alert('erreur');</script>"""
            return self.notifyErrors(errors)
        
        # if child form
        if is_childform:
            tmp = TemporaryDocument(self.getParentDatabase(), self, REQUEST)
            tmp.setItem("Plomino_Parent_Field", parent_field)
            tmp.setItem("Plomino_Parent_Form", parent_form)
            return self.ChildForm(temp_doc=tmp)
            
        doc = db.createDocument()
        doc.setItem('Form', self.getFormName())
        
        # execute the onCreateDocument code of the form
        valid = ''
        valid = self.runFormulaScript("form_"+self.id+"_oncreate", doc, self.onCreateDocument)
        if valid is None or valid=='':
            doc.saveDocument(REQUEST, True)
        else:
            db.writeMessageOnPage(valid, REQUEST, '', False)
            REQUEST.RESPONSE.redirect(db.absolute_url())

    security.declarePublic('getFields')
    def getFields(self, includesubforms=False, doc=None):
        """get fields
        """
        fieldlist = self.portal_catalog.search({'portal_type' : ['PlominoField'], 'path': '/'.join(self.getPhysicalPath())})
        result = [f.getObject() for f in fieldlist]
        result.sort(key=lambda elt: elt.id.lower())
        if includesubforms:
            for subformname in self.getSubforms(doc):
                result=result+self.getParentDatabase().getForm(subformname).getFields(True)
        return result

    security.declarePublic('getHidewhenFormulas')
    def getHidewhenFormulas(self):
        """Get hidden formulae
        """
        list = self.portal_catalog.search({'portal_type' : ['PlominoHidewhen'], 'path': '/'.join(self.getPhysicalPath())})
        return [h.getObject() for h in list]

    security.declarePublic('getActions')
    def getActions(self, target, hide=True):
        """Get actions
        """
        all = self.portal_catalog.search({'portal_type' : ['PlominoAction'], 'path': '/'.join(self.getPhysicalPath())})

        filtered = []
        for a in all:
            obj_a=a.getObject()
            if hide:
                try:
                    #result = RunFormula(target, obj_a.getHidewhen())
                    result = self.runFormulaScript("action_"+self.id+"_"+obj_a.id+"_hidewhen", target, obj_a.Hidewhen)
                except Exception:
                    #if error, we hide anyway
                    result = True
                if not result:
                    filtered.append(obj_a)
            else:
                filtered.append(obj_a)
        return filtered

    security.declarePublic('getFormName')
    def getFormName(self):
        """Return the form name
        """
        return self.id

    security.declarePublic('getParentDatabase')
    def getParentDatabase(self):
        """Get the database containing this form
        """
        return self.getParentNode()


    security.declareProtected(READ_PERMISSION, 'displayDocument')
    def displayDocument(self,doc,editmode=False, creation=False, subform=False, request=None):
        """display the document using the form's layout
        """
        
        # remove the hidden content
        html_content = self.applyHideWhen(doc)
        
        #if editmode, we had a hidden field to handle the Form item value
        if editmode and not subform:
            html_content = "<input type='hidden' name='Form' value='"+self.getFormName()+"' />" + html_content

        # insert the fields with proper value and rendering
        for field in self.getFields():
            fieldName = field.id
            fieldblock='<span class="plominoFieldClass">'+fieldName+'</span>'
            if creation and not(fieldblock in html_content) and request is not None:
                html_content = "<input type='hidden' name='"+fieldName+"' value='"+str(request.get(fieldName,''))+"' />" + html_content
            if fieldblock in html_content:
                html_content = html_content.replace(fieldblock, field.getFieldRender(self, doc, editmode, creation, request=request))

        # insert subforms
        for subformname in self.getSubforms(doc):
            subformrendering=self.getParentDatabase().getForm(subformname).displayDocument(doc, editmode, creation, subform=True, request=request)
            html_content = html_content.replace('<span class="plominoSubformClass">'+subformname+'</span>', subformrendering)

        # insert the actions
        if doc is None:
            target = self
        else:
            target = doc
        actionsToDisplay=[a.id for a in self.getActions(target, True)]
        for action in self.getActions(target, False):
            actionName = action.id
            if actionName in actionsToDisplay:
                actionDisplay = action.ActionDisplay
                pt=self.getRenderingTemplate(actionDisplay+"Action")
                if pt is None:
                    pt=self.getRenderingTemplate("LINKAction")
                action_render = pt(plominoaction=action, plominotarget=target)
            else:
                action_render=''
            html_content = html_content.replace('<span class="plominoActionClass">'+actionName+'</span>', action_render)

        return html_content

    security.declareProtected(READ_PERMISSION, 'childDocument')
    def childDocument(self,doc):
        """
        """
        parent_form = self.getParentDatabase().getForm(doc.Plomino_Parent_Form)
        parent_field = parent_form.getFormField(doc.Plomino_Parent_Field)
        fields = parent_field.getSettings().field_mapping.split(',')
        
        raw_values = []
        for f in fields:
            v = doc.getItem(f)
            if hasattr(v, 'strftime'):
                raw_values.append(DateToString(doc.getItem(f), self.getParentDatabase().getDateTimeFormat()))
            else:
                raw_values.append(v)
        
        html = """<div id="raw_values">%s</div>""" % json.dumps(raw_values)
        
        html = html + """<div id="parent_field">%s</div>""" % doc.Plomino_Parent_Field
        
        for f in fields:
            html = html + """<span id="%s" plomino="1">%s</span>""" % (f, doc.getRenderedItem(f, form=self))
        
        return html
                        
    security.declareProtected(READ_PERMISSION, 'applyHideWhen')
    def applyHideWhen(self, doc=None):
        """evaluate hide-when formula and return resulting layout
        """
        plone_tools = getToolByName(self, 'plone_utils')
        encoding = plone_tools.getSiteEncoding()
        html_content = self.getField('FormLayout').getRaw(self).decode(encoding)
        html_content = html_content.replace('\n', '')

        # remove the hidden content
        for hidewhen in self.getHidewhenFormulas():
            hidewhenName = hidewhen.id
            try:
                if doc is None:
                    target = self
                else:
                    target = doc
                result = self.runFormulaScript("hidewhen_"+self.id+"_"+hidewhen.id+"_formula", target, hidewhen.Formula)
            except Exception:
                #if error, we hide anyway
                result = True
            start = '<span class="plominoHidewhenClass">start:'+hidewhenName+'</span>'
            end = '<span class="plominoHidewhenClass">end:'+hidewhenName+'</span>'
            
            if getattr(self, 'isDynamicHidewhen', False):
                if result:
                    style = ' style="display: none"'
                else:
                    style = ''
                html_content = re.sub(start,'<div id="' + hidewhenName + '"' + style + '>', html_content)
                html_content = re.sub(end,'</div>', html_content)
            else:
                if result:
                    regexp = start+'.+?'+end
                    html_content = re.sub(regexp,'', html_content)
                else:
                    html_content = html_content.replace(start, '')
                    html_content = html_content.replace(end, '')
                
        return html_content
    
    security.declareProtected(READ_PERMISSION, 'getHidewhenAsJSON')
    def getHidewhenAsJSON(self, REQUEST):
        """
        """
        result = {}
        target = TemporaryDocument(self.getParentDatabase(), self, REQUEST)
        for hidewhen in self.getHidewhenFormulas():
            try:
                isHidden = self.runFormulaScript("hidewhen_"+self.id+"_"+hidewhen.id+"_formula", target, hidewhen.Formula)
            except Exception:
                #if error, we hide anyway
                isHidden = True
            result[hidewhen.id] = isHidden 
        
        return json.dumps(result)
                
    security.declarePublic('formLayout')
    def formLayout(self, request=None):
        """return the form layout in edit mode (used to compose a new
        document)
        """
        return self.displayDocument(None, True, True, request=request)

    security.declarePublic('openBlankForm')
    def openBlankForm(self, request=None):
        """check beforeCreateDocument then open the form
        """
        # execute the beforeCreateDocument code of the form
        valid = ''
        if hasattr(self,'beforeCreateDocument') and self.beforeCreateDocument is not None:
            valid = self.runFormulaScript("form_"+self.id+"_beforecreate", self, self.beforeCreateDocument)
        if valid is None or valid=='' or self.hasDesignPermission(self):
            return self.displayDocument(None, True, True, request=request)
        else:
            self.REQUEST.RESPONSE.redirect(self.getParentDatabase().absolute_url()+"/ErrorsMessages?disable_border=1&error="+valid)
    
    security.declarePublic('at_post_edit_script')
    def at_post_edit_script(self):
        """clean up the layout before saving
        """
        self.cleanFormulaScripts("form_"+self.id)

    security.declarePublic('getFormField')
    def getFormField(self, fieldname):
        """return the field
        """
        return getattr(self, fieldname, None)

    security.declarePublic('hasDateTimeField')
    def hasDateTimeField(self):
        """return true if the form contains at least one DateTime field
        or a datagrid (as a datagrid may contain a date)
        """
        fields=self.getFields(includesubforms=True)
        for f in fields:
            if f.getFieldType() in ["DATETIME", "DATAGRID"]:
                return True
        return False

    security.declarePublic('hasGoogleVisualizationField')
    def hasGoogleVisualizationField(self):
        """return true if the form contains at least one GoogleVisualization field
        """
        fields=self.getFields(includesubforms=True)
        for f in fields:
            if f.getFieldType() == "GOOGLEVISUALIZATION":
                return True
        return False
    
    security.declarePublic('getSubforms')
    def getSubforms(self, doc=None):
        """return the names of the subforms embedded in the form
        """
        html_content = html_content = self.applyHideWhen(doc)
        r = re.compile('<span class="plominoSubformClass">([^<]+)</span>')
        return [i.strip() for i in r.findall(html_content)]

    security.declarePublic('readInputs')
    def readInputs(self, doc, REQUEST, process_attachments=False):
        """ read submitted values in REQUEST and store them in document according
        fields definition
        """
        for f in self.getFields(includesubforms=True, doc=doc):
            mode = f.getFieldMode()
            fieldName = f.id
            if mode=="EDITABLE":
                submittedValue = REQUEST.get(fieldName)
                if submittedValue is not None:
                    if submittedValue=='':
                        doc.removeItem(fieldName)
                    else:
                        v = f.processInput(submittedValue, doc, process_attachments)
                        doc.setItem(fieldName, v)
                else:
                    #the field was not submitted, probably because it is not part of the form (hide-when, ...)
                    #so we just let it unchanged, but with SELECTION or DOCLINK, we need to presume it was empty
                    #(as SELECT/checkbox/radio tags do not submit an empty value, they are just missing
                    #in the querystring)
                    fieldtype = f.getFieldType()
                    if fieldtype == "SELECTION" or fieldtype == "DOCLINK":
                        doc.removeItem(fieldName)
                    

    security.declareProtected(READ_PERMISSION, 'searchDocuments')
    def searchDocuments(self,REQUEST):
        """search documents in the view matching the submitted form fields values
        """
        db = self.getParentDatabase()
        searchview = db.getView(self.getSearchView())

        #index search
        index = db.getIndex()
        query={'PlominoViewFormula_'+searchview.getViewName() : True}

        for f in self.getFields(includesubforms=True):
            fieldName = f.id
            #if fieldName is not an index -> search doesn't matter and returns all
            submittedValue = REQUEST.get(fieldName)
            if submittedValue is not None:
                if not submittedValue=='':
                    # if non-text field, convert the value
                    if f.getFieldType()=="NUMBER":
                        v = long(submittedValue)
                    elif f.getFieldType()=="FLOAT":
                        v = float(submittedValue)
                    elif f.getFieldType()=="DATETIME":
                        v = submittedValue
                    else:
                        v = submittedValue
                    query[fieldName]=v
        results=index.dbsearch(query, None)

        #filter search with searchformula
        searchformula=self.getSearchFormula()
        if searchformula:
            filteredResults = []
            for doc in results:
                try:
                    valid = self.runFormulaScript("form_"+self.id+"_searchformula", doc.getObject(), self.SearchFormula)
                except Exception, e:
                    valid = False
                if valid:
                    filteredResults.append(doc)
            results = filteredResults

        return self.OpenForm(searchresults=results)

    security.declarePublic('validateInputs')
    def validateInputs(self, REQUEST, doc=None):
        errors=[]
        for f in self.getFields(includesubforms=True, doc=doc):
            fieldname = f.id
            fieldtype = f.getFieldType()
            submittedValue = REQUEST.get(fieldname)

            # STEP 1: check mandatory fields
            if submittedValue is None or submittedValue=='':
                if f.getMandatory()==True:
                    errors.append(fieldname+" "+PlominoTranslate("is mandatory",self))
            else:
                # STEP 2: check data types
                errors = errors + f.validateFormat(submittedValue)
                
        if len(errors)==0:
            # STEP 3: check validation formula
            tmp = TemporaryDocument(self.getParentDatabase(), self, REQUEST, doc)
            for f in self.getFields(includesubforms=True):
                formula = f.getValidationFormula()
                if not formula=='':
                    s=''
                    try:
                        s = self.runFormulaScript("field_"+self.id+"_"+f.id+"_ValidationFormula", tmp, f.ValidationFormula)
                    except:
                        pass
                    if not s=='':
                        errors.append(s)

        return errors

    security.declarePublic('notifyErrors')
    def notifyErrors(self, errors):
        return self.ErrorsMessages(errors=errors)

    security.declarePublic('getPosition')
    def getPosition(self):
        """Return the form position in the database
        """
        try :
            return self.Position
        except Exception :
            return None

    security.declarePublic('isNewDocument')
    def isNewDocument(self):
        """
        """
        # when the context is a form, it is necessarily a new doc
        return True
        
    security.declarePublic('isEditMode')
    def isEditMode(self):
        """
        """
        # if REQUEST exists, test the current command
        if hasattr(self, 'REQUEST'):
            command=self.REQUEST.URL.split('/')[-1].lower()
            return command in ['openform', 'edit']
        else:
            return False
    
registerType(PlominoForm, PROJECTNAME)
# end of class PlominoForm

##code-section module-footer #fill in your manual code here
##/code-section module-footer






# -*- coding: utf-8 -*-
#
# File: PlominoAccessControl.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'

from DateTime import DateTime
from time import strptime
from Products.CMFCore.utils import getToolByName
import htmlentitydefs as entity
import urllib
import csv
from cStringIO import StringIO
import Missing

def DateToString(d, format='%d/%m/%Y'):
    """return the date as string using the given format, default is '%d/%m/%Y'
    """
    #return DateTime(*d[0:6]).strftime(format)
    return d.strftime(format)

def StringToDate(str_d, format='%d/%m/%Y'):
    """parse the string using the given format (default is '%d/%m/%Y') and return the date 
    """
    dt = strptime(str_d, format)
    if len(dt)>=5:
        return DateTime(dt[0], dt[1], dt[2], dt[3], dt[4])
    else:
        return DateTime(dt[0], dt[1], dt[2])

def DateRange(d1, d2):
    """return all the days from the d1 date to the d2 date (included)
    """
    duration = int(d2-d1)
    result=[]
    current=d1
    for d in range(duration+1):
        result.append(current)
        current = current+1
    return result

def Now():
    """ current date and tile
    """
    return DateTime()

def sendMail(db, recipients, title, html_message, sender=None):
    """Send an email
    """
    host = getToolByName(db, 'MailHost')
    plone_tools = getToolByName(db, 'plone_utils')
    message = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n'
    message = message + "<html>"
    message = message + html_message
    message = message + "</html>"
    if sender is None:
        sender = db.getCurrentUser().getProperty("email")
    encoding = plone_tools.getSiteEncoding()
    host.secureSend(message, recipients,
        sender, subject=title,
        subtype='html', charset=encoding,
        debug=False, From=sender)
    
def userFullname(db, userid):
    """ return user fullname if exist, else return userid, and return Unknown if user not found
    """
    user=getToolByName(db, 'portal_membership').getMemberById(userid)
    if not(user is None):
        fullname=user.getProperty('fullname')
        if fullname=='':
            return userid
        else:
            return fullname
    else:
        return "Unknown"
        
def userInfo(db, userid):
    """ return user object
    """
    user=getToolByName(db, 'portal_membership').getMemberById(userid)
    return user

def PlominoTranslate(message, context, domain='CMFPlomino'):
    """
    """
    plone_tools = getToolByName(context, 'plone_utils')
    encoding = plone_tools.getSiteEncoding()
    translation_service = getToolByName(context, 'translation_service')
    if isinstance(message, Exception):
        try:
            message = message[0]
        except (TypeError, IndexError):
            pass     
    msg = translation_service.translate(msgid=message, domain=domain)
    return msg.encode(encoding) # convert unicode to site encoding

def htmlencode(s):
    """ replace unicode characters with their corresponding html entities
    """
    
    t=""
    for i in s:
        if ord(i) in entity.codepoint2name:
            name = entity.codepoint2name.get(ord(i))
            entityCode = entity.name2codepoint.get(name)
            t +="&#" + str(entityCode)
        else:
            t+=i
    return t

def urlencode(h):
    """ call urllib.urlencode
    """
    return urllib.urlencode(h)

def asList(x):
    """ if not list, return x in a single-element list
    """
    if hasattr(x, 'append'):
        return x
    else:
        return [x]

def csv_to_array(csvcontent, delimiter='\t', quotechar='"'):
    """ csvcontent might be a string or a file
    """
    if csvcontent is None:
        return []
    if type(csvcontent) is str:
        csvfile = StringIO(csvcontent)
    else:
        csvfile = csvcontent
    return [l for l in csv.reader(csvfile, delimiter=delimiter, quotechar=quotechar)]

def open_url(url, asFile=False):
    """ retrieve content from url
    """
    f=urllib.urlopen(url)
    if asFile:
        return f.fp
    else:
        return f.read()

def MissingValue():
    """ Useful to test search results value (as Missing.Value cannot be imported in scripts)
    """
    return Missing.Value



from zope.interface import Interface, implements
from zope.formlib import form
from plone.app.portlets.portlets import base
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope import schema
from Products.CMFPlomino.browser import PloneMessageFactory as _

class IPortlet(Interface):
    pass

class Assignment(base.Assignment):
    implements(IPortlet)

    title = u'Plomino design portlet'

class Renderer(base.Renderer):
    render = ViewPageTemplateFile('design_portlet.pt')
    
    def parentdatabase(self):
        """
        """
        parentdb = self.context.unrestrictedTraverse(self.context.getPhysicalPath())
        while len(parentdb.getPhysicalPath())>0 and parentdb.__class__.__name__!="PlominoDatabase":
            parentdb = parentdb.getParentNode()
        return parentdb

    def currenttype(self):
        """
        """
        return self.context.__class__.__name__
    
    def hasDesignPermission(self):
        """
        """
        return self.parentdatabase().hasDesignPermission()
    
    @property
    def available(self):
        return self.hasDesignPermission()
    
class AddForm(base.AddForm):
    form_fields = form.Fields(IPortlet)

    def create(self, data):
        return Assignment()


class IElementPortlet(Interface):
    """Contains the template used to fill a template form
    """
    header = schema.TextLine(title=_(u"Portlet header"),
                             description=_(u"Title of the rendered portlet"),
                             required=True)
    db_path = schema.TextLine(title=_(u"Database path"),
                             description=_(u"Path of the database where the element is stored"),
                             required=True)
    element_id = schema.TextLine(title=_(u"Element ID"),
                             description=_(u"ID of the form to be displayed"),
                             required=True)
    

class ElementPortletAssignment(base.Assignment):
    """Initialises a Plomino Element portlet
    """
    implements(IElementPortlet)
    
    title = u'Plomino Element Display'
    header = u""
    db_path = ""
    element_id = ""
    
    def __init__(self, header=u"", db_path="", element_id=""):
        self.header = header
        self.db_path = db_path
        self.element_id = element_id

    
class ElementPortletRenderer(base.Renderer):
    """Displays the Plomino Element portlet
    """
    render = ViewPageTemplateFile('element_portlet.pt')
    
    def getElement(self):
        """Get the element to be displayed by the portlet
        """
        db = self.context.restrictedTraverse(self.data.db_path.encode(), None)
        if db:
            element = getattr(db, self.data.element_id.encode(), None)
            if hasattr(element, 'formLayout'):
                return element
        return None
        
    @property
    def available(self):
        return True
    
    @property
    def hasGoogleVisualizationField(self):
        element = self.getElement() 
        if element is not None:
            return element.hasGoogleVisualizationField()
        else:
            return False
        
    @property
    def action_url(self):
        element = self.getElement()
        base_url = element.absolute_url()
        if element.isSearchForm:
            return base_url+"/searchDocuments"
        if element.isPage:
            return "."
        return base_url+"/createDocument"
    
    def elementLayout(self):
        """Get the element layout to be displayed by the portlet
        """
        #import pdb; pdb.set_trace()
        element = self.getElement()
        if element:
            return element.formLayout(self.request)
        else:
            return """<p>The database cannot be found or the element cannot be displayed.</p>"""
    
class ElementPortletAddForm(base.AddForm):
    """Creates a portlet used to display a Plomino element everywhere in a Plone site
    """
    label = _(u"Add a Plomino Element Portlet")
    description = _(u"This portlet displays an element of a Plomino database.")
    form_fields = form.Fields(IElementPortlet)

    def create(self, data):
        return ElementPortletAssignment(**data)
    
class ElementPortletEditForm(base.EditForm):
    """Edit a portlet used to display a Plomino element everywhere in a Plone site
    """
    label = _(u"Edit a Plomino Element Portlet")
    description = _(u"This portlet displays an element of a Plomino database.")
    form_fields = form.Fields(IElementPortlet)



# -*- coding: utf-8 -*-
#
# File: PlominoAccessControl.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.CMFPlomino.config import *
from Products.CMFCore import permissions
from AccessControl.PermissionRole import rolesForPermissionOn
from AccessControl import getSecurityManager
from Products.CMFCore.utils import getToolByName
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Persistence import Persistent
        
class PlominoAccessControl(Persistent):
    """Plomino access control utilities
    """
    security = ClassSecurityInfo()

    PLOMINO_PERMISSIONS = {
    "NoAccess" : [],
    "PlominoReader" : [
        READ_PERMISSION,
        permissions.View,
        permissions.AccessContentsInformation],
    "PlominoAuthor" : [
        READ_PERMISSION,
        EDIT_PERMISSION,
        REMOVE_PERMISSION,
        CREATE_PERMISSION,
        ADD_CONTENT_PERMISSION,
        permissions.View,
        permissions.AccessContentsInformation,
        #permissions.AddPortalContent,
        #permissions.ModifyPortalContent
        ],
    "PlominoEditor" : [
        READ_PERMISSION,
        EDIT_PERMISSION,
        REMOVE_PERMISSION,
        CREATE_PERMISSION,
        ADD_CONTENT_PERMISSION,
        permissions.View,
        permissions.AccessContentsInformation,
        #permissions.AddPortalContent,
        #permissions.ModifyPortalContent
        ],
    "PlominoDesigner" : [
        READ_PERMISSION,
        EDIT_PERMISSION,
        REMOVE_PERMISSION,
        CREATE_PERMISSION,
        DESIGN_PERMISSION,
        ADD_CONTENT_PERMISSION,
        ADD_DESIGN_PERMISSION,
        permissions.View,
        permissions.AccessContentsInformation,
        permissions.AddPortalContent,
        permissions.ModifyPortalContent],
    "PlominoManager" : [
        READ_PERMISSION,
        EDIT_PERMISSION,
        REMOVE_PERMISSION,
        CREATE_PERMISSION,
        DESIGN_PERMISSION,
        ADD_CONTENT_PERMISSION,
        ADD_DESIGN_PERMISSION,
        ACL_PERMISSION,
        permissions.View,
        permissions.AccessContentsInformation,
        permissions.AddPortalContent,
        permissions.ModifyPortalContent]
    }
    
    PLOMINO_RIGHTS_PRIORITY = ["NoAccess",
                               "PlominoReader",
                               "PlominoAuthor",
                               "PlominoEditor",
                               "PlominoDesigner",
                               "PlominoManager",
                               "Owner"]

    # Methods

    security.declarePublic('__init__')
    def __init__(self):
        """
        """
        self.ACL_initialized=0
        self.UserRoles={}

    security.declarePublic('getUserRoles')
    def getUserRoles(self):
        """return the db Plomino roles
        """
        return self.UserRoles.keys()

    security.declarePublic('getUsersForRoles')
    def getUsersForRoles(self,role):
        """return the users having the given Plomino user role
        """
        if self.UserRoles.has_key(role):
            return self.UserRoles[role]
        else:
            return ''

    security.declarePublic('hasUserRole')
    def hasUserRole(self,user,role):
        """test if the given user has the given Plomino user role
        """
        if self.UserRoles.has_key(role):
            role_people = self.UserRoles[role].keys()
            if user in role_people:
                return True
            else:
                groupstool = self.portal_groups
                usergroups = [g.id for g in groupstool.getGroupsByUserId(user)]
                test = False
                for u in role_people:
                    if u in usergroups:
                        test = True
                return test
        else:
            return False

    security.declarePublic('getUsersForRight')
    def getUsersForRight(self,right):
        """return the users having the given Plomino access right
        """
        return self.users_with_local_role(right)

    security.declarePublic('getCurrentUser')
    def getCurrentUser(self):
        """get the current user
        """
        membershiptool = getToolByName(self, 'portal_membership')
        return membershiptool.getAuthenticatedMember()

    security.declarePublic('getCurrentUserRights')
    def getCurrentUserRights(self):
        """get the current user Plomino rights
        """
        try:
            userid = self.getCurrentUser().getMemberId()
            rights = self.get_local_roles_for_userid(userid)
            if len(rights)==0:
                # no specific rights for this user, we first check group rights
                groupstool = self.portal_groups
                usergroups = [g.id for g in groupstool.getGroupsByUserId(userid)]
                for g in usergroups:
                    rights = rights + self.get_local_roles_for_userid(g)
            if len(rights)==0:
                # still no specific rights, so return AuthenticatedAccessRight
                default_right = getattr(self, "AuthenticatedAccessRight", "NoAccess")
                rights = [default_right]
            return rights
        except Exception:
            return [getattr(self, "AnomynousAccessRight", "NoAccess")]

    security.declarePublic('hasCurrentUserRight')
    def hasCurrentUserRight(self, right):
        """ test if current user as the given right or upper
        """
        rights = self.getCurrentUserRights()
        levels = [self.PLOMINO_RIGHTS_PRIORITY.index(r) for r in rights if r in self.PLOMINO_RIGHTS_PRIORITY]
        if len(levels)>0:
            maxlevel = max(levels)
            return maxlevel >= self.PLOMINO_RIGHTS_PRIORITY.index(right)
        else:
            return False
        
    security.declarePublic('getCurrentUserRoles')
    def getCurrentUserRoles(self):
        """get current user roles
        """
        userid = self.getCurrentUser().getUserName()
        allroles = self.getUserRoles()
        roles = []
        for r in allroles:
            if self.hasUserRole(userid, r):
                roles.append(r)
        return roles

    security.declarePublic('isCurrentUserAuthor')
    def isCurrentUserAuthor(self,doc):
        """is the current user the document's author?
        """
        # the user must at least have edit permission
        if not self.checkUserPermission(EDIT_PERMISSION):
            return False
        
        # if the user is Owner or Manager, no problem
        general_plone_rights = self.getCurrentUser().getRolesInContext(doc)
        if 'Owner' in general_plone_rights or 'Manager' in general_plone_rights:
            return True
        
        # check if the user is more powerful than a regular PlominoAuthor
        current_rights = self.getCurrentUserRights()
        if "PlominoEditor" in current_rights or "PlominoDesigner" in current_rights or "PlominoManager" in current_rights:
            return True
        
        # if he is just a PlominoAuthor, check if he is author of this very document
        if 'PlominoAuthor' in current_rights:
            authors = doc.getItem('Plomino_Authors')
            if authors is None:
                authors=[]
            name = self.getCurrentUser().getUserName()
            if '*' in authors:
                return True
            if name in authors:
                return True
            roles = self.getCurrentUserRoles()
            for r in roles:
                if r in authors:
                    return True
            groupstool = self.portal_groups
            usergroups = [g.id for g in groupstool.getGroupsByUserId(name)]
            for u in authors:
                if u in usergroups:
                    return True
            return False
        
        return False

    security.declarePublic('hasReadPermission')
    def hasReadPermission(self, obj=None):
        """has read permission?
        """
        if obj is None:
            result = self.checkUserPermission(READ_PERMISSION)
        else:
            result = getSecurityManager().checkPermission(READ_PERMISSION, obj)==True
        return result
    
    security.declarePublic('hasCreatePermission')
    def hasCreatePermission(self, obj=None):
        """has create permission ?
        """
        if obj is None:
            obj = self
        return getSecurityManager().checkPermission(CREATE_PERMISSION, obj)==True and getSecurityManager().checkPermission(ADD_CONTENT_PERMISSION, obj)==True
    
    security.declarePublic('hasEditPermission')
    def hasEditPermission(self, obj=None):
        """has edit permission ?
        """
        if obj is None:
            obj = self
        return getSecurityManager().checkPermission(EDIT_PERMISSION, obj)==True
    
    security.declarePublic('hasRemovePermission')
    def hasRemovePermission(self, obj=None):
        """has remove permission ?
        """
        if obj is None:
            obj = self
        return getSecurityManager().checkPermission(REMOVE_PERMISSION, obj)==True
        
    security.declarePublic('hasDesignPermission')
    def hasDesignPermission(self, obj=None):
        """has design permission ?
        """
        if obj is None:
            obj = self
        return getSecurityManager().checkPermission(DESIGN_PERMISSION, obj)==True
    
    security.declarePublic('initializeACL')
    def initializeACL(self):
        """create the default Plomino access rights
        """
        self._addRole("PlominoReader")
        self.setPlominoPermissions("PlominoReader", "PlominoReader")
        self._addRole("PlominoAuthor")
        self.setPlominoPermissions("PlominoAuthor", "PlominoAuthor")
        self._addRole("PlominoEditor")
        self.setPlominoPermissions("PlominoEditor", "PlominoEditor")
        self._addRole("PlominoDesigner")
        self.setPlominoPermissions("PlominoDesigner", "PlominoDesigner")
        self._addRole("PlominoManager")
        self.setPlominoPermissions("PlominoManager", "PlominoManager")
        self.setPlominoPermissions("Manager", "PlominoManager")
        self.setPlominoPermissions("Owner", "PlominoManager")
        self.AnomynousAccessRight="NoAccess"
        self.setPlominoPermissions("Anonymous", "NoAccess")
        self.AuthenticatedAccessRight="NoAccess"
        self.setPlominoPermissions("Authenticated", "NoAccess")
        self.ACL_initialized=1

    security.declarePrivate('setPlominoPermissions')
    def setPlominoPermissions(self,right,p):
        """Note: be careful, Zope 'roles' are different than Plomino
        'roles'. Plomino access rights are implemented using Zope roles
        and Plomino roles are just a dictionary handled by Plomino db
        with no impact in the Zope security
        """
        self.manage_role(right, permissions=self.PLOMINO_PERMISSIONS[p])

    security.declarePublic('checkUserPermission')
    def checkUserPermission(self, perm, obj=None):
        """check user's permission
        """
        if obj is None:
            obj = self
        return getSecurityManager().checkPermission(perm, obj)

    security.declareProtected(ACL_PERMISSION, 'addACLEntry')
    def addACLEntry(self,REQUEST):
        """add an entry in the ACL
        """
        user=REQUEST.get('newuser')
        accessright=REQUEST.get('accessright')
        self.manage_setLocalRoles(user, [accessright])
        REQUEST.RESPONSE.redirect('./DatabaseACL')

    security.declareProtected(ACL_PERMISSION, 'removeACLEntries')
    def removeACLEntries(self,REQUEST):
        """remove entries in the ACL
        """
        users=REQUEST.get('users')
        if type(users)==str:
            users = [users]
        self.manage_delLocalRoles(users)
        REQUEST.RESPONSE.redirect('./DatabaseACL')

    security.declareProtected(ACL_PERMISSION, 'setGenericAccess')
    def setGenericAccess(self,REQUEST):
        """set the generic users access rights
        """
        anonymousaccessright=REQUEST.get('anonymousaccessright')
        self.AnomynousAccessRight=anonymousaccessright
        self.setPlominoPermissions("Anonymous", anonymousaccessright)

        authenticatedaccessright=REQUEST.get('authenticatedaccessright')
        self.AuthenticatedAccessRight=authenticatedaccessright
        self.setPlominoPermissions("Authenticated", authenticatedaccessright)
        
        # update Plone workflow mappings
        workflow_tool = getToolByName(self, 'portal_workflow')
        wfs = workflow_tool.getWorkflowsFor(self)
        for wf in wfs:
            if not isinstance( wf, DCWorkflowDefinition ):
                continue
            wf.updateRoleMappingsFor( self )
            
        REQUEST.RESPONSE.redirect('./DatabaseACL')

    security.declareProtected(ACL_PERMISSION, 'addPlominoUserRole')
    def addPlominoUserRole(self,REQUEST):
        """add a user role in the ACL
        """
        newrole=REQUEST.get('newrole')
        # roles names must be enclosed in brackets wo they can be distinguished
        # from users names in Authors fields
        newrole='['+newrole+']'
        roles = self.UserRoles
        if not(roles.has_key(newrole)):
            roles[newrole] = {}
            self.UserRoles = roles
        REQUEST.RESPONSE.redirect('./DatabaseACL')

    security.declareProtected(ACL_PERMISSION, 'removePlominoUserRole')
    def removePlominoUserRole(self,REQUEST):
        """remove a user role from the ACL
        """
        role=REQUEST.get('role')
        roles = self.UserRoles
        if roles.has_key(role):
            del roles[role]
            self.UserRoles = roles
        REQUEST.RESPONSE.redirect('./DatabaseACL')

    security.declareProtected(ACL_PERMISSION, 'addPlominoRoleToUser')
    def addPlominoRoleToUser(self,REQUEST):
        """give a role to a user
        """
        role=REQUEST.get('role')
        user=REQUEST.get('user')
        roles = self.UserRoles
        if roles.has_key(role):
            userslist = roles[role]
            userslist[user]=1
            roles[role] = userslist
            self.UserRoles = roles
        REQUEST.RESPONSE.redirect('./DatabaseACL')

    security.declareProtected(ACL_PERMISSION, 'removePlominoRoleFromUser')
    def removePlominoRoleFromUser(self,REQUEST):
        """
        """
        role=REQUEST.get('role')
        user=REQUEST.get('user')
        roles = self.UserRoles
        if roles.has_key(role):
            userslist = roles[role]
            if userslist.has_key(user):
                del userslist[user]
                roles[role] = userslist
                self.UserRoles = roles
        REQUEST.RESPONSE.redirect('./DatabaseACL')

    security.declarePublic('getPortalMembers')
    def getPortalMembers(self):
        """return all members
        """
        membershiptool = getToolByName(self, 'portal_membership')
        #return membershiptool.listMemberIds ()
        return membershiptool.searchForMembers()
        
    security.declarePublic('getPortalMembersIds')
    def getPortalMembersIds(self):
        """return all members id
        """
        l=self.getPortalMembers()
        return [m.getId() for m in l]
    
    security.declarePublic('getPortalGroups')
    def getPortalGroups(self):
        """return all groups
        """
        groupstool = self.portal_groups
        return groupstool.listGroups()
        
    security.declarePublic('getPortalMembersGroupsIds')
    def getPortalMembersGroupsIds(self):
        tab = []
        """return all members id
        """
        l=self.getPortalMembers()
        tab = tab + l
        """return all groups id
        """
        g=self.getPortalGroups()
        tab = tab + g
        return tab
    
    security.declarePublic('getSpecificRights')
    def getSpecificRights(self, key=None):
        """
        """
        if not hasattr(self, 'specific_rights'):
            self.specific_rights = {'specific_deletedocument': 'PlominoAuthor'}
        if key is None:
            return self.specific_rights
        else:
            return self.specific_rights.get(key, None)

    security.declarePublic('setSpecificRights')
    def setSpecificRights(self, right, key):
        """
        """
        s = self.getSpecificRights()
        s[right] = key
        self.specific_rights = s
            
    security.declareProtected(ACL_PERMISSION, 'manage_specificrights')
    def manage_specificrights(self, REQUEST):
        """
        """
        if REQUEST.get('specific_deletedocument', None) is not None:
            self.setSpecificRights('specific_deletedocument', REQUEST.get('specific_deletedocument'))
        REQUEST.RESPONSE.redirect('./DatabaseACL')
    
    security.declareProtected(ACL_PERMISSION, 'getWorkflowStates')
    def getWorkflowStates(self):
        """
        """  
        ob = self
        wftool = self._getWorkflowTool()
        states = {}
        if wftool is not None:
            wf_ids = wftool.getChainFor(ob)
            for wf_id in wf_ids:
                wf = wftool.getWorkflowById(wf_id)
                if wf is not None:
                    # XXX a standard API would be nice
                    if hasattr(wf, 'getReviewStateOf'):
                        # Default Workflow
                        state = wf.getReviewStateOf(ob)
                    elif hasattr(wf, '_getWorkflowStateOf'):
                        # DCWorkflow
                        state = wf._getWorkflowStateOf(ob, id_only=1)
                    else:
                        state = '(Unknown)'
                    states[wf_id] = state
        return states
                    
    security.declareProtected(READ_PERMISSION, 'checkReadPermission')
    def checkReadPermission(self):
        """called from templates to check access permission
        """
        pass

    security.declareProtected(EDIT_PERMISSION, 'checkEditPermission')
    def checkEditPermission(self):
        """called from templates to check access permission
        """
        pass
    
    security.declareProtected(REMOVE_PERMISSION, 'checkRemovePermission')
    def checkRemovePermission(self):
        """called from templates to check access permission
        """
        pass
    
    security.declareProtected(CREATE_PERMISSION, 'checkCreatePermission')
    def checkCreatePermission(self):
        """called from templates to check access permission
        """
        pass
    
    security.declareProtected(DESIGN_PERMISSION, 'checkDesignPermission')
    def checkDesignPermission(self):
        """called from templates to check access permission
        """
        pass
    
    security.declareProtected(ACL_PERMISSION, 'checkACLPermission')
    def checkACLPermission(self):
        """called from templates to check access permission
        """
        pass
    



# -*- coding: utf-8 -*-
#
# File: PlominoDatabase.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces
from Products.ATContentTypes.content.folder import ATFolder
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.CMFPlomino.config import *

##code-section module-header #fill in your manual code here
from Products.Archetypes.public import *
from Products.Archetypes.utils import make_uuid

from zope import event
from zope.app.container.contained import ObjectRemovedEvent
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from OFS.Folder import *
from OFS.ObjectManager import ObjectManager
from Products.CMFPlomino.PlominoUtils import *
import string
import Globals
import transaction

from index.PlominoIndex import PlominoIndex
from PlominoAccessControl import PlominoAccessControl
from PlominoDesignManager import PlominoDesignManager
from PlominoReplicationManager import PlominoReplicationManager
from PlominoScheduler import PlominoScheduler

from Products.CMFCore.PortalFolder import PortalFolderBase as PortalFolder

##/code-section module-header

schema = Schema((

    TextField(
        name='AboutDescription',
        allowable_content_types=('text/html',),
        widget=RichWidget(
            label="About this database",
            description="Describe the database, its objectives, its targetted audience, etc...",
            label_msgid='CMFPlomino_label_AboutDescription',
            description_msgid='CMFPlomino_help_AboutDescription',
            i18n_domain='CMFPlomino',
        ),
        default_output_type="text/html",
    ),
    TextField(
        name='UsingDescription',
        allowable_content_types=('text/html',),
        widget=RichWidget(
            label="Using this database",
            description="Describe how to use the database",
            label_msgid='CMFPlomino_label_UsingDescription',
            description_msgid='CMFPlomino_help_UsingDescription',
            i18n_domain='CMFPlomino',
        ),
        default_output_type="text/html",
    ),
    BooleanField(
        name='IndexAttachments',
        default=False,
        widget=BooleanField._properties['widget'](
            label="Index file attachments",
            description="If enabled, files attached in File Attachments fields will be indexed. It might increase the index size.",
            label_msgid='CMFPlomino_label_IndexAttachments',
            description_msgid='CMFPlomino_help_IndexAttachments',
            i18n_domain='CMFPlomino',
        ),
    ),
    BooleanField(
        name='FulltextIndex',
        default=False,
        widget=BooleanField._properties['widget'](
            label="Local full-text index",
            description="If enabled, documents are full-text indexed into the Plomino index.",
            label_msgid='CMFPlomino_label_FulltextIndex',
            description_msgid='CMFPlomino_help_FulltextIndex',
            i18n_domain='CMFPlomino',
        ),
    ),
    BooleanField(
        name='debugMode',
        default=False,
        widget=BooleanField._properties['widget'](
            label="Debug mode",
            description="If enabled, script and formula errors are logged.",
            label_msgid='CMFPlomino_label_debugMode',
            description_msgid='CMFPlomino_help_debugMode',
            i18n_domain='CMFPlomino',
        ),
    ),
    BooleanField(
        name='StorageAttachments',
        default=False,
        widget=BooleanField._properties['widget'](
            label="Use File System Storage for attachments",
            description="File System Storage must be installed on the portal",
            label_msgid='CMFPlomino_label_StorageAttachments',
            description_msgid='CMFPlomino_help_StorageAttachments',
            i18n_domain='CMFPlomino',
        ),
    ),
    BooleanField(
        name='CountDocuments',
        default=False,
        widget=BooleanField._properties['widget'](
            label="Count the number of documents",
            description="If enabled, count the number of documents for each view. Display might be slower.",
            description_msgid="CMFPlomino_help_ShowDocumentsNumbers",
            label_msgid="CMFPlomino_label_ShowDocumentsNumbers",
            i18n_domain='CMFPlomino',
        ),
    ),
    StringField(
        name='DateTimeFormat',
        default="%Y/%m/%d",
        widget=StringField._properties['widget'](
            label='Date/time format',
            description='Format example: %Y-%m-%d',
            label_msgid='CMFPlomino_label_DateTimeFormat',
            description_msgid="CMFPlomino_help_DateTimeFormat",
            i18n_domain='CMFPlomino',
        ),
    ),
    StringField(
        name='StartPage',
        widget=StringField._properties['widget'](
            label='Start page',
            description='Element to display instead of the regular database menu.',
            label_msgid='CMFPlomino_label_StartPage',
            description_msgid='CMFPlomino_help_StartPage',
            i18n_domain='CMFPlomino',
        ),
    ),
    BooleanField(
        name='DoNotListUsers',
        default=False,
        widget=BooleanField._properties['widget'](
            label="Do not list portal users",
            description="If True, in ACL screen, users are entered using a free text field, if False, using a selection list. Use True when the amount of users is big.",
            description_msgid="CMFPlomino_help_DoNotListUsers",
            label_msgid="CMFPlomino_label_DoNotListUsers",
            i18n_domain='CMFPlomino',
        ),
    ),
    BooleanField(
        name='DoNotReindex',
        default=False,
        widget=BooleanField._properties['widget'](
            label="Do not re-index documents on design changes",
            description="If True, documents are not re-indexed automatically when views, columns or indexed fields are changed. Note: manual refresh db is then needed.",
            description_msgid="CMFPlomino_help_DoNotReindex",
            label_msgid="CMFPlomino_label_DoNotReindex",
            i18n_domain='CMFPlomino',
        ),
    ),
),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

PlominoDatabase_schema = getattr(ATFolder, 'schema', Schema(())).copy() + \
    getattr(PlominoAccessControl, 'schema', Schema(())).copy() + \
    getattr(PlominoDesignManager, 'schema', Schema(())).copy() + \
    getattr(PlominoReplicationManager, 'schema', Schema(())).copy() + \
    getattr(PlominoScheduler, 'schema', Schema(())).copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class PlominoDatabase(ATFolder, PlominoAccessControl, PlominoDesignManager, PlominoReplicationManager, PlominoScheduler):
    """
    """
    security = ClassSecurityInfo()
    implements(interfaces.IPlominoDatabase)

    meta_type = 'PlominoDatabase'
    _at_rename_after_creation = True

    schema = PlominoDatabase_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    security.declarePublic('__init__')
    def __init__(self,oid,**kw):
        """
        """
        ATFolder.__init__(self, oid, **kw)
        self.plomino_version = VERSION
        self.setStatus("Ready")
        PlominoAccessControl.__init__(self)

    security.declarePublic('at_post_create_script')
    def at_post_create_script(self):
        """DB initialization
        """
        self.initializeACL()
        index = PlominoIndex(FULLTEXT=self.FulltextIndex)
        self._setObject(index.getId(), index)
        resources = Folder('resources')
        resources.title='resources'
        self._setObject('resources', resources)
        scripts = Folder('scripts')
        scripts.title='scripts'
        self._setObject('scripts', scripts)

    security.declarePublic('getStatus')
    def getStatus(self):
        """return DB current status
        """
        return getattr(self, "plomino_db_status", "Ready")
        
    security.declarePublic('setStatus')
    def setStatus(self, status, commit=False):
        """set DB current status
        """
        if commit:
            txn = transaction.get()
            self.plomino_db_status = status
            txn.commit()
        else:
            self.plomino_db_status = status
            
    security.declarePublic('checkBeforeOpenDatabase')
    def checkBeforeOpenDatabase(self):
        """check if custom start page
        """
        if self.checkUserPermission(READ_PERMISSION):
            if hasattr(self, 'REQUEST') and not self.checkUserPermission(DESIGN_PERMISSION):
                self.REQUEST["disable_border"]=True
            try:
                if self.StartPage:        
                    if hasattr(self, self.getStartPage()):
                        target = getattr(self, self.getStartPage())
                    return getattr(target, target.defaultView())()
                else:
                    return self.OpenDatabase()
            except:
                return self.OpenDatabase()
        else:
            raise Unauthorized, "You cannot read this content"
        
    security.declarePublic('getForms')
    def getForms(self, sortbyid=False):
        """return the database forms list
        """
        form_list = self.portal_catalog.search({'portal_type' : ['PlominoForm'], 'path': '/'.join(self.getPhysicalPath())})
        form_obj_list = [a.getObject() for a in form_list]
        if sortbyid:
            form_obj_list.sort(key=lambda elt: elt.id.lower())
        else:
            form_obj_list.sort(key=lambda elt: elt.getPosition())
        return form_obj_list

    security.declarePublic('getViews')
    def getViews(self, sortbyid=False):
        """return the database views list
        """
        view_list = self.portal_catalog.search({'portal_type' : ['PlominoView'], 'path': '/'.join(self.getPhysicalPath())})
        view_obj_list = [a.getObject() for a in view_list]
        if sortbyid:
            view_obj_list.sort(key=lambda elt: elt.id.lower())
        else:
            view_obj_list.sort(key=lambda elt: elt.getPosition())
        return view_obj_list

    security.declarePublic('getAgents')
    def getAgents(self):
        """return the database agents list
        """
        agent_list = self.portal_catalog.search({'portal_type' : ['PlominoAgent'], 'path': '/'.join(self.getPhysicalPath())})
        agent_obj_list = [a.getObject() for a in agent_list]
        agent_obj_list.sort(key=lambda elt: elt.id.lower())
        return agent_obj_list

    security.declarePublic('getForm')
    def getForm(self,formname):
        """return a PlominoForm
        """
        obj = getattr(self, formname, None)
        if obj is not None and obj.Type() == 'PlominoForm':
            return obj
        else:
            return None

    security.declarePublic('getView')
    def getView(self,viewname):
        """return a PlominoView
        """
        obj = getattr(self, viewname, None)
        if obj is not None and obj.Type() == 'PlominoView':
            return obj
        else:
            return None

    security.declareProtected(CREATE_PERMISSION, 'createDocument')
    def createDocument(self):
        """create a unique ID and invoke PlominoDocument factory
        """
        newid = make_uuid()
        self.invokeFactory( type_name='PlominoDocument', id=newid)
        doc = getattr(self, newid)
        return doc

    security.declarePublic('getDocument')
    def getDocument(self, docid):
        """return a PlominoDocument
        """
        if "/" in docid:
            # let's assume it is a path
            docid = docid.split("/")[-1]
        return getattr(self, docid, None)

    security.declareProtected(REMOVE_PERMISSION, 'deleteDocument')
    def deleteDocument(self,doc):
        """delete the document from database
        """
        if not self.isCurrentUserAuthor(doc):
            raise Unauthorized, "You cannot delete this document."
        else:
            # execute the onDeleteDocument code of the form
            try:
                form = doc.getForm()
                self.runFormulaScript("form_"+form.id+"_ondelete", doc, form.onDeleteDocument)
            except Exception:
                pass

            self.getIndex().unindexDocument(doc)
            event.notify(ObjectRemovedEvent(doc, self, doc.id))
            return PortalFolder.manage_delObjects(self, doc.id, None)

    security.declareProtected(REMOVE_PERMISSION, 'deleteDocuments')
    def deleteDocuments(self,ids=None, massive=True):
        """delete documents from database
        if massive, onDelete formula and index updating are not performed (use refreshDB to update)
        """
        if ids is None:
            ids=[doc.id for doc in self.getAllDocuments()]
            
        if massive:
            ObjectManager.manage_delObjects(self, ids)
        else:
            for id in ids:
                try:
                    self.deleteDocument(self.getDocument(id))
                except:
                    # if insufficient access rights, we continue
                    pass
            
    security.declareProtected(REMOVE_PERMISSION, 'manage_deleteDocuments')
    def manage_deleteDocuments(self, REQUEST):
        """delete documents action
        """
        strids = REQUEST.get('deldocs', None)
        if strids is not None:
            ids = [i for i in strids.split('@') if i is not '']
            self.deleteDocuments(ids=ids, massive=False)
        REQUEST.RESPONSE.redirect('.')
        
    security.declarePublic('getIndex')
    def getIndex(self):
        """return the database index
        """
        return getattr(self, 'plomino_index')

    security.declarePublic('getAllDocuments')
    def getAllDocuments(self):
        """return all the database documents
        """
        #return [d.getObject() for d in self.portal_catalog.search({'portal_type' : ['PlominoDocument'], 'path': '/'.join(self.getPhysicalPath())})]
        index = self.getIndex()
        res = index.dbsearch({},None)
        return [d.getObject() for d in res]

    security.declarePublic('isDocumentsCountEnabled')
    def isDocumentsCountEnabled(self):
        """
        """
        try :
            return self.CountDocuments
        except Exception :
            return False


registerType(PlominoDatabase, PROJECTNAME)
# end of class PlominoDatabase

##code-section module-footer #fill in your manual code here
##/code-section module-footer






# -*- coding: utf-8 -*-
#
# File: googlechart.py
#
# Copyright (c) 2009 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.com>"""
__docformat__ = 'plaintext'

from zope.formlib import form
from zope.interface import implements
from zope.schema import getFields
from zope.schema import TextLine

from dictionaryproperty import DictionaryProperty

from Products.Five.formlib.formbase import EditForm

from base import IBaseField, BaseField

class IGooglechartField(IBaseField):
    """
    Google chart field schema
    """
    editrows = TextLine(title=u'Rows',
                      description=u'Size of the editable text area',
                      default=u"6",
                      required=False)

class GooglechartField(BaseField):
    """
    """
    implements(IGooglechartField)

    def validate(self, submittedValue):
        """
        """
        errors=[]
        # no validation needed (we do not want to parse the GoogleChart param)
        return errors
    
    def processInput(self, submittedValue):
        """
        """
        lines = submittedValue.replace('\r', '').split('\n')
        params = {}
        for l in lines:
            if "=" in l:
                (key, value) = l.split('=')
            else:
                key = l
                value = ''
            params[key] = value
        return params

    
for f in getFields(IGooglechartField).values():
    setattr(GooglechartField, f.getName(), DictionaryProperty(f, 'parameters'))

class SettingForm(EditForm):
    """
    """
    form_fields = form.Fields(IGooglechartField)
    


# -*- coding: utf-8 -*-
#
# File: PlominoAgent.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT and Xavier PERROT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.CMFPlomino.config import *

##code-section module-header #fill in your manual code here
from Products.Archetypes.public import *
from AccessControl.SecurityManagement import getSecurityManager, setSecurityManager, newSecurityManager
import transaction
##/code-section module-header

schema = Schema((

    StringField(
        name='id',
        widget=StringField._properties['widget'](
            label="Id",
            description="The agent id",
            label_msgid='CMFPlomino_label_agent_id',
            description_msgid='CMFPlomino_help_agent_id',
            i18n_domain='CMFPlomino',
        ),
    ),
    TextField(
        name='Content',
        widget=TextAreaWidget(
            label="Code",
            description="Code to execute",
            label_msgid="CMFPlomino_label_AgentContent",
            description_msgid="CMFPlomino_help_AgentContent",
            i18n_domain='CMFPlomino',
        ),
    ),
    BooleanField(
        name='Scheduled',
        widget=BooleanField._properties['widget'](
            label="Scheduled",
            description="Scheduled agent",
            label_msgid='CMFPlomino_label_Scheduled',
            description_msgid='CMFPlomino_help_Scheduled',
            i18n_domain='CMFPlomino',
        ),
    ),
    StringField(
        name='AgentUser',
        widget=StringField._properties['widget'](
            label="Agent user",
            description="Effective user used to run the agent",
            label_msgid='CMFPlomino_label_AgentUser',
            description_msgid='CMFPlomino_help_AgentUser',
            i18n_domain='CMFPlomino',
        ),
    ),
    StringField(
        name='Cron',
        default='* 1 * * *',
        widget=StringField._properties['widget'](
            label="Cron configuration",
            description="Format : crontab UNIX",
            label_msgid='CMFPlomino_label_Cron',
            description_msgid='CMFPlomino_help_Cron',
            i18n_domain='CMFPlomino',
        ),
    ),
),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

PlominoAgent_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class PlominoAgent(BaseContent, BrowserDefaultMixin):
    """
    """
    security = ClassSecurityInfo()
    implements(interfaces.IPlominoAgent)

    meta_type = 'PlominoAgent'
    _at_rename_after_creation = True

    schema = PlominoAgent_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    security.declarePublic('at_post_create_script')
    def at_post_create_script(self):
        """
        """
        db = self.getParentDatabase()
        db.managePlominoCronTab()

    security.declarePublic('at_post_edit_script')
    def at_post_edit_script(self):
        """
        """
        self.cleanFormulaScripts("agent_"+self.id)
        #cron tab
        db = self.getParentDatabase()
        db.managePlominoCronTab()

    security.declarePublic('manage_beforeDelete')
    def manage_beforeDelete(self,item,container):
        """
        """
        #cron tab
        db = self.getParentDatabase()
        db.managePlominoCronTab(True)

    security.declarePublic('getParentDatabase')
    def getParentDatabase(self):
        """
        """
        return self.getParentNode()

    def __call__(self, *args):
        """
        """
        txn = transaction.get()
        plominoContext = self
        try:
            result = self.runFormulaScript("agent_"+self.id, plominoContext, self.Content, True, *args)
            txn.commit()
        except Exception, e:
            txn.abort()
            result = "Error: %s \nCode->\n%s" % (e, self.Content())
            
        return result
        
    security.declarePublic('runAgent')
    def runAgent(self,REQUEST=None):
        """execute the python code
        """

        #acl = self.acl_users
        #newSecurityManager(None, acl.getUserById(str(self.AgentUser), None))
        txn = transaction.get()
        plominoContext = self
        plominoReturnURL = self.getParentDatabase().absolute_url()
        try:
            #RunFormula(plominoContext, "agent_"+self.id, self.Content())
            r=self.runFormulaScript("agent_"+self.id, plominoContext, self.Content)
            txn.commit()
            if (REQUEST != None) and (REQUEST.get('REDIRECT', None)== "True"):
                if r is not None:
                    plominoReturnURL=r
                REQUEST.RESPONSE.redirect(plominoReturnURL)
        except Exception, e:
            txn.abort()
            return "Error: %s \nCode->\n%s" % (e, self.Content())


registerType(PlominoAgent, PROJECTNAME)
# end of class PlominoAgent

##code-section module-footer #fill in your manual code here
##/code-section module-footer






from setuptools import setup, find_packages
import os

version = '1.7'

setup(name='Plomino',
      version=version,
      description="Create specific applications in Plone without developping. Created by Makina Corpus.",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
      keywords='RAD, dynamic forms, Lotus Domino',
      author='Eric BREHAULT',
      author_email='eric.brehault@makina-corpus.org',
      url='http://www.plomino.net',
      license='ZPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'simplejson',
      ],
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """
      )



from Products.CMFPlomino.fields.text import ITextField
from Products.CMFPlomino.fields.selection import ISelectionField
from Products.CMFPlomino.fields.number import INumberField
from Products.CMFPlomino.fields.datetime import IDatetimeField
from Products.CMFPlomino.fields.name import INameField
from Products.CMFPlomino.fields.doclink import IDoclinkField

import logging
logger = logging.getLogger('Plomino migration')

def migrate(db):
    """
    """
    messages = []
    if not(hasattr(db, "plomino_version")):
        msg = migrate_to_130(db)
        messages.append(msg)
    if db.plomino_version=="1.3.0":
        # no migration needed here
        db.plomino_version = "1.4.0"
    if db.plomino_version=="1.4.0":
        msg = migrate_to_15(db)
        messages.append(msg)
    if db.plomino_version=="1.5":
        msg = migrate_to_16(db)
        messages.append(msg)
    if db.plomino_version=="1.6":
        msg = migrate_to_161(db)
        messages.append(msg)
    if db.plomino_version=="1.6.1":
        # no migration needed here
        db.plomino_version = "1.6.2"
    if db.plomino_version=="1.6.2":
        # no migration needed here
        db.plomino_version = "1.6.3"
    if db.plomino_version=="1.6.3":
        msg = migrate_to_164(db)
        messages.append(msg)
    if db.plomino_version=="1.6.4":
        msg = migrate_to_17(db)
        messages.append(msg)
    
    return messages

def migrate_to_130(db):
    """ PlominoField schema has been changed, all type-specific parameters
    have been removed and are now handled through specific adapters.
    """
    db.setDebugMode(False)
    msg = "Migration to 1.3: DebugMode attribute added"
    
    for form in db.getForms():
        for field in form.getFields():
            type = field.getFieldType()
            if type == "TEXT":
                adapt = ITextField(field)
                adapt.widget = "TEXT"
            elif type == "NUMBER":
                adapt = INumberField(field)
                adapt.type = "INTEGER"
            elif type == "FLOAT":
                field.setFieldType("NUMBER")
                adapt = INumberField(field)
                adapt.type = "FLOAT"
            elif type == "NAME":
                adapt = INameField(field)
                adapt.type = "SINGLE"
            elif type == "NAMES":
                field.setFieldType("NAME")
                adapt = INameField(field)
                adapt.type = "MULTI"
            elif type in ("SELECTION", "MULTISELECTION", "CHECKBOX", "RADIO"):
                field.setFieldType("SELECTION")
                adapt = ISelectionField(field)
                l = getattr(field, "SelectionList", None)
                if l is not None:
                    adapt.selectionlist = list(l)
                v = getattr(field, "SelectionListFormula", None)
                if v is not None:
                    adapt.selectionlistformula = v.raw
                adapt.separator = getattr(field, "DisplayModList", getattr(field, "OtherDisplayMod", None))
                if type == "SELECTION":
                    adapt.widget = "SELECT"
                elif type == "MULTISELECTION":
                    adapt.widget = "MULTISELECT"
                elif type == "CHECKBOX":
                    adapt.widget = "CHECKBOX"
                elif type == "RADIO":
                    adapt.widget = "RADIO"
            elif type == "DOCLINK":
                adapt = IDoclinkField(field)
                v = getattr(field, "SelectionListFormula", None)
                if v is not None:
                    adapt.documentslistformula = v.raw
    msg = msg + ", FieldType remapped" 
    db.plomino_version = "1.3.0"
    return msg
    
def migrate_to_15(db):
    """ new attribute in Column: DisplaySum
    """
    for v_obj in db.getViews():
        for c in v_obj.getColumns():
            c.setDisplaySum(False)
    msg = "Migration to 1.5: DisplaySum attribute added"
    db.plomino_version = "1.5"
    return msg

def migrate_to_16(db):
    """ attribute Position in column is replaced by AtFolder sorting
    """
    for v_obj in db.getViews():
        # sort columns by their Position
        orderedcolumns = []
        for c in v_obj.getColumns():
            if not(c is None):
                orderedcolumns.append([c.Position, c])
        orderedcolumns.sort()
        
        # set the position using the previous sorting
        for i, c in enumerate(orderedcolumns):
            v_obj.moveObject(c[1].id, i)
            v_obj.plone_utils.reindexOnReorder(v_obj)
            
    msg = "Migration to 1.6: Position column attribute deleted"
    db.plomino_version = "1.6"
    return msg

def migrate_to_161(db):
    """ Fulltext index db attribute.
    """
    db.setFulltextIndex(False)
    msg = "Migration to 1.6.1: Fulltext index db attribute set to False"
    db.plomino_version = "1.6.1"
    return msg

def migrate_to_164(db):
    """ Fulltext index db attribute.
    """
    db.setDoNotReindex(False)
    msg = "Migration to 1.6.4: DoNotReindex db attribute set to False"
    db.plomino_version = "1.6.4"
    return msg

def migrate_to_17(db):
    """ dynamic hidewhen attribute
    """
    for form in db.getForms():
        form.setIsDynamicHidewhen(False)
    msg = "Migration to 1.7: Dynamic hide-when initialized"
    db.plomino_version = "1.7"
    return msg


## Script (Python) "getNotAddableTypes"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
# customize this script to filter addable portal types based on
# context, the current user or other criteria
return ('Favorite', 'PlominoDocument',)



# -*- coding: utf-8 -*-
#
# File: base.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
#
# Zope Public License (ZPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from zope.interface import Interface, implements
from zope.annotation.interfaces import IAnnotations
from persistent.dict import PersistentDict

import simplejson as json

class IBaseField(Interface):
    """
    """

class BaseField(object):
    """
    """
    implements(IBaseField)
    
    def __init__(self, context):
        """Initialize adapter."""
        self.context = context
        # get annotation to store param values
        annotations = IAnnotations(context)
        self.parameters = annotations.get("PLOMINOFIELDCONFIG", None)
        if not self.parameters:
            annotations["PLOMINOFIELDCONFIG"] = PersistentDict()
            self.parameters = annotations["PLOMINOFIELDCONFIG"]
        # allow access
        self.__allow_access_to_unprotected_subobjects__ = True
            
    def validate(self, strValue):
        """
        """
        errors=[]
        return errors
    
    def processInput(self, strValue):
        """
        """
        if type(strValue) == str:
            strValue = strValue.decode('utf-8')
        return strValue
    
    def getSelectionList(self, doc):
        """
        """
        return None
    
    def getFieldValue(self, form, doc, editmode, creation, request):
        """
        """
        fieldName = self.context.id
        mode = self.context.getFieldMode()
        
        db = self.context.getParentDatabase()
        if doc is None:
            target = form
        else:
            target = doc
        
        fieldValue = None
        if mode=="EDITABLE":
            if doc is None:
                if creation and not self.context.Formula()=="":
                    fieldValue = db.runFormulaScript("field_"+form.id+"_"+fieldName+"_formula", target, self.context.Formula)
                elif request is None:
                    fieldValue = ""
                else:
                    row_data_json = request.get("Plomino_datagrid_rowdata", None)
                    if row_data_json is not None:
                        # datagrid form case
                        parent_form = request.get("Plomino_Parent_Form", None)
                        parent_field = request.get("Plomino_Parent_Field", None)
                        data = json.loads(row_data_json)
                        datagrid_fields = db.getForm(parent_form).getFormField(parent_field).getSettings().field_mapping.split(',')
                        fieldValue = data[datagrid_fields.index(fieldName)]
                    else: 
                        fieldValue = request.get(fieldName, '')
            else:
                fieldValue = doc.getItem(fieldName)

        if mode=="DISPLAY" or mode=="COMPUTED":
            fieldValue = db.runFormulaScript("field_"+form.id+"_"+fieldName+"_formula", target, self.context.Formula)

        if mode=="CREATION":
            if creation:
                # Note: on creation, there is no doc, we use self as param
                # in formula
                fieldValue = db.runFormulaScript("field_"+form.id+"_"+fieldName+"_formula", form, self.context.Formula)
            else:
                fieldValue = doc.getItem(fieldName)
        
        if fieldValue is None:
            fieldValue = ""
        return fieldValue